import re

import html2text
from parsel import Selector

from dataharvest.purifier.purifier import BasePurifier
from dataharvest.schema import Document


class WechatPurifier(BasePurifier):
    def __init__(self):
        self.convertor = html2text.HTML2Text()
        self.convertor.ignore_links = True
        self.convertor.body_width = 0

    def match(self, url: str) -> bool:
        return "/weixin.qq.com/s/" in url

    def purify(self, doc: Document) -> Document:
        selector = Selector(doc.page_content)

        # 清洗无用标签
        selector.xpath("//section[@class='mp_profile_iframe_wrp']").drop()

        # 标题
        title_label = selector.xpath("//h1[@id='activity-name']").get()
        title = self.convertor.handle(title_label)

        # 内容
        content_label = selector.xpath("//div[@id='js_content']").get()
        content_label_replaced = re.sub(r"<img\b[^>]*?>", "[图片]", content_label)

        content = self.convertor.handle(content_label_replaced)

        return Document(
            url=doc.url, metadata={**doc.metadata}, page_content=title + content
        )



from dataharvest.spider.base import SpiderConfig
from dataharvest.spider.common_spider import CommonSpider
from dataharvest.spider.mafengwo_spider import MaFengWoSpider
from dataharvest.spider.spider import AutoSpider, BaseSpider
from dataharvest.spider.toutiao_spider import ToutiaoSpider
from dataharvest.spider.ssl_spider import SslSpider
from dataharvest.spider.xiaohongshu_spider import XiaoHongShuSpider

__all__ = [
    "AutoSpider",
    "CommonSpider",
    "BaseSpider",
    "ToutiaoSpider",
    "MaFengWoSpider",
    "SpiderConfig",
    "SslSpider",
    "XiaoHongShuSpider"
]



from typing import Optional

import httpx

from dataharvest.schema import Document
from dataharvest.spider.base import SpiderConfig
from dataharvest.spider.spider import BaseSpider


class SslSpider(BaseSpider):
    def __init__(self, config: Optional[SpiderConfig] = None):
        super().__init__(config)
        self.client = httpx.Client(**BaseSpider.convert_2_httpx_client_arg(config))
        self.a_client = httpx.AsyncClient(
            **BaseSpider.convert_2_httpx_client_arg(config)
        )
        self._config = self._merge_config(config)

        if not self._config.headers:
            self._config.headers = {
                "user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            }

    def match(self, url: str) -> bool:
        return "/www.360doc.com/content/" in url

    def crawl(self, url: str, config: Optional[SpiderConfig] = None) -> Document:
        res = self.client.get(url, headers=self._config.headers, follow_redirects=True)
        res.raise_for_status()
        document = Document(
            url=str(res.request.url),
            metadata={},
            page_content=res.content.decode("utf-8"),
        )
        return document

    async def a_crawl(
            self, url: str, config: Optional[SpiderConfig] = None
    ) -> Document:
        res = await self.a_client.get(
            url, headers=self._config.headers, follow_redirects=True
        )
        res.raise_for_status()
        document = Document(
            url=str(res.request.url),
            metadata={},
            page_content=res.content.decode("utf-8"),
        )
        return document



from dataharvest.purifier import AutoPurifier
from dataharvest.schema import Document
from dataharvest.spider import AutoSpider


class DataHarvest:
    def __init__(self):
        self.spider = AutoSpider()
        self.purifier = AutoPurifier()

    async def a_crawl_and_purify(self, url) -> Document:
        doc = await self.spider.a_crawl(url)
        return self.purifier.purify(doc)

    def crawl_and_purify(self, url) -> Document:
        doc = self.spider.crawl(url)
        return self.purifier.purify(doc)



from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Dict, Optional, Union as Uni

from dataharvest.proxy.base import BaseProxy
from dataharvest.schema import Document


@dataclass
class SpiderConfig:
    proxy_gene_func: Optional[BaseProxy] = None
    headers: Uni[Dict[str, str], None] = None


class BaseSpider(ABC):
    index = 0
    _config: Optional[SpiderConfig] = None

    def __init__(self, config: Optional[SpiderConfig] = None):
        self._config = self._merge_config(config)

    def _merge_config(self, config: Optional[SpiderConfig]) -> Optional[SpiderConfig]:
        if self._config is None and config is None:
            return SpiderConfig()

        if config is None:
            return self._config

        if self._config is None:
            return config

        return SpiderConfig(
            proxy_gene_func=config.proxy_gene_func or self._config.proxy_gene_func,
            headers=config.headers or self._config.headers,
        )

    @staticmethod
    def convert_2_playwright_lunch_arg(config: SpiderConfig):
        proxy_dict = None
        if config.proxy_gene_func:
            proxy_obj = config.proxy_gene_func()
            if proxy_obj:
                proxy_dict = {
                    "server": f"{proxy_obj.protocol}://{proxy_obj.host}:{proxy_obj.port}",
                    "username": proxy_obj.username,
                    "password": proxy_obj.password,
                }
        return {"proxy": proxy_dict}

    @staticmethod
    def convert_2_httpx_client_arg(config: Optional[SpiderConfig]):
        if config is None:
            return {}
        proxy = None
        if config.proxy_gene_func:
            proxy_obj = config.proxy_gene_func()

            if proxy_obj:
                if proxy_obj.username and proxy_obj.password:
                    proxy = f"{proxy_obj.protocol}://{proxy_obj.username}:{proxy_obj.password}@{proxy_obj.host}:{proxy_obj.port}"
                else:
                    proxy = f"{proxy_obj.protocol}://{proxy_obj.host}:{proxy_obj.port}"

        return {"proxy": proxy or None, "headers": config.headers or None}

    @abstractmethod
    def match(self, url: str) -> bool:
        raise NotImplementedError

    @abstractmethod
    def crawl(self, url: str, config: Optional[SpiderConfig] = None) -> Document:
        raise NotImplementedError

    @abstractmethod
    async def a_crawl(
            self, url: str, config: Optional[SpiderConfig] = None
    ) -> Document:
        raise NotImplementedError



from typing import Optional

from playwright.async_api import async_playwright
from playwright.sync_api import sync_playwright
from playwright_stealth import stealth_async, stealth_sync

from dataharvest.schema import Document
from dataharvest.spider import BaseSpider, SpiderConfig
from dataharvest.spider.utils import random_user_agent


class XiaoHongShuSpider(BaseSpider):

    def __init__(self, config: Optional[SpiderConfig] = None):
        super().__init__(config)
        self._cookies = [{
            'name': "webId",
            'value': "xxx123",
            'domain': ".xiaohongshu.com",
            'path': "/"
        }]

    def match(self, url: str) -> bool:
        return "/xhslink.com/" in url

    def crawl(self, url: str, config: Optional[SpiderConfig] = None) -> Document:
        config = self._merge_config(config)
        with sync_playwright() as playwright:
            browser = playwright.chromium.launch(headless=False,
                                                 **self.convert_2_playwright_lunch_arg(
                                                     config))
            browser_context = browser.new_context(
                user_agent=random_user_agent()
            )

            browser_context.add_cookies(self._cookies)

            context_page = browser_context.new_page()

            stealth_sync(context_page)
            context_page.goto(url)
            context_page.wait_for_load_state(state="load")
            html = context_page.content()
            document = Document(url=context_page.url, metadata={}, page_content=html)
            return document

    async def a_crawl(
            self, url: str, config: Optional[SpiderConfig] = None
    ) -> Document:
        config = self._merge_config(config)
        async with async_playwright() as playwright:
            browser = await playwright.chromium.launch(
                **self.convert_2_playwright_lunch_arg(config))
            browser_context = await browser.new_context(
                user_agent=random_user_agent()
            )

            await browser_context.add_cookies(self._cookies)

            context_page = await browser_context.new_page()
            await stealth_async(context_page)
            await context_page.goto(url)
            await context_page.wait_for_load_state(state="load")
            html = await context_page.content()
            document = Document(url=context_page.url, metadata={}, page_content=html)
            return document



from abc import ABC, abstractmethod

from dataharvest.schema import SearchResult


class BaseSearcher(ABC):
    @abstractmethod
    def search(self, keyword: str, **kwargs) -> SearchResult:
        raise NotImplementedError



import asyncio

from dataharvest.base import DataHarvest
from dataharvest.searcher import TavilySearcher


def test_pipline():
    searcher = TavilySearcher()
    dh = DataHarvest()
    r = searcher.search("战国水晶杯")
    tasks = [dh.a_crawl_and_purify(item.url) for item in r.items]
    loop = asyncio.get_event_loop()
    result = loop.run_until_complete(asyncio.gather(*tasks))

    assert len(result) > 0



from abc import ABC, abstractmethod

from dataharvest.schema import Document


class BasePurifier(ABC):
    index = 0

    @abstractmethod
    def match(self, url: str) -> bool:
        raise NotImplementedError

    @abstractmethod
    def purify(self, doc: Document) -> Document:
        raise NotImplementedError



from dataclasses import dataclass, field

from typing import List, Optional


@dataclass
class Document:
    url: str
    metadata: dict
    page_content: str


@dataclass
class SearchResultItem:
    title: str
    url: str
    score: Optional[float] = 0.0
    description: str = ""
    content: str = ""


@dataclass
class SearchResult:
    keyword: str
    answer: str
    items: List[SearchResultItem]
    images: List[str] = field(default_factory=list)



import hashlib
import json
import time
from typing import Optional

import httpx

from dataharvest.schema import SearchResult
from dataharvest.schema import SearchResultItem
from dataharvest.searcher.base import BaseSearcher


class SkySearcher(BaseSearcher):
    """
    天工搜索
    Reference: https://model-platform.tiangong.cn/api-reference
    """

    def __init__(self, app_key: str, app_secret: str, base_url: Optional[str] = None):
        self.url = base_url or "https://api.singularity-ai.com/sky-saas-search/api/v1/search"
        self.app_key = app_key
        self.app_secret = app_secret

    def _get_headers(self):
        timestamp = str(int(time.time()))
        sign_content = self.app_key + self.app_secret + timestamp
        sign_result = hashlib.md5(sign_content.encode("utf-8")).hexdigest()

        return {
            "app_key": self.app_key,
            "timestamp": timestamp,
            "sign": sign_result,
            "Content-Type": "application/json",
        }

    @staticmethod
    def parse_response(keyword, response) -> SearchResult:
        items = []
        answer = ""
        for line in response.iter_lines():
            if line.startswith('{"code":400007,"code_msg":"服务器忙，请稍后再试"}'):
                raise Exception("服务器忙，请稍后再试")
            elif line.startswith(
                    """data: {"type":1,"card_type":"search_result","target":"finish"""
            ):
                search_result_json = json.loads(line[line.index("{"):])
                search_result = search_result_json["arguments"][0]["messages"][0][
                    "sourceAttributions"
                ]
                items = [
                    SearchResultItem(url=i["seeMoreUrl"],
                                     title=i["title"],
                                     description=i["snippet"])
                    for i in search_result]
            elif line.startswith(
                    """data: {"type":1,"card_type":"markdown","target":"finish"""
            ):
                markdown_json = json.loads(line[line.index("{"):])
                answer += markdown_json["arguments"][0]["messages"][0]["text"]

        return SearchResult(keyword=keyword, answer=answer, items=items)

    def search(self, keyword: str, **kwargs) -> SearchResult:
        data = {
            "content": keyword,
            "stream_resp_type": kwargs.get("stream_resp_type", "all"),
        }

        response = httpx.post(self.url, headers=self._get_headers(), json=data)
        response.raise_for_status()
        return self.parse_response(keyword, response)



import os
from typing import Optional

import httpx

from dataharvest.schema import SearchResult, SearchResultItem
from dataharvest.searcher.base import BaseSearcher


class TavilySearcher(BaseSearcher):
    """
    Reference: https://docs.tavily.com/docs/tavily-api/rest_api
    """

    def __init__(self, api_key: Optional[str] = None, max_results: int = 5, **kwargs):
        """
        :param api_key:  Your Tavily API key.
        :param keyword:  The search query string.
        :param search_depth:  The depth of the search.
        It can be basic or advanced.
        Default is basic for quick results and advanced for indepth high quality results but longer response time.
        Advanced calls equals 2 requests.
        :param include_images:  Include a list of query related images in the response.
        Default is False.
        :param include_answer:  Include answers in the search results.
        Default is False.
        :param include_raw_content:  Include raw content in the search results.
        Default is False.
        :param max_results:  The number of maximum search results to return.
        Default is 5.
        :param include_domains:  A list of domains to specifically include in the search results.
        Default is None, which includes all domains.
        :param exclude_domains:  A list of domains to specifically exclude from the search results.
        Default is None, which doesn't exclude any domains.
        :param api_key:
        :param max_results:
        :param kwargs:
        """
        tavily_api_key = api_key or os.getenv("TAVILY_API_KEY")

        if not tavily_api_key:
            raise ValueError("Tavily API key is required.")

        self.api_key = tavily_api_key
        self.max_results = max_results
        self.search_depth = kwargs.get("search_depth", "basic")
        self.include_answer = kwargs.get("include_answer", False)
        self.include_images = kwargs.get("include_images", False)
        self.include_raw_content = kwargs.get("include_raw_content", False)
        self.include_domains = kwargs.get("include_domains", [])
        self.exclude_domains = kwargs.get("exclude_domains", [])

        self.client = httpx.Client()

    def search(self, keyword: str, **kwargs) -> SearchResult:
        """
        search a keyword,you can customize parameters in kwargs

        :return:  A SearchResult object.
        """
        req = {
            "api_key": self.api_key,
            "query": keyword,
            "search_depth": kwargs.get("search_depth", self.search_depth),
            "include_answer": kwargs.get("include_answer", self.include_answer),
            "include_images": kwargs.get("include_images", self.include_images),
            "include_raw_content": kwargs.get(
                "include_raw_content", self.include_raw_content
            ),
            "max_results": kwargs.get("max_results", self.max_results),
            "include_domains": kwargs.get("include_domains", self.include_domains),
            "exclude_domains": kwargs.get("exclude_domains", self.exclude_domains),
        }

        res = self.client.post("https://api.tavily.com/search", json=req)
        res.raise_for_status()
        res_json = res.json()
        images = res_json.get("images", [])
        answer = res_json.get("answer", "")
        items = [
            SearchResultItem(
                title=item["title"],
                url=item["url"],
                score=item["score"],
                description=item["content"],
            )
            for item in res_json["results"]
        ]
        return SearchResult(keyword, answer, images, items)



import pytest

from dataharvest.spider.common_spider import CommonSpider
from dataharvest.spider.spider import AutoSpider


def test_common_spider():
    spider = CommonSpider()
    doc = spider.crawl(
        "https://baike.baidu.com/item/%E9%87%91%E7%BC%95%E7%8E%89%E8%A1%A3/617831?fr=ge_ala"
    )
    print(doc)


def test_auto_spider():
    spider = AutoSpider()
    doc = spider.crawl(
        "https://baike.baidu.com/item/%E9%87%91%E7%BC%95%E7%8E%89%E8%A1%A3/617831?fr=ge_ala"
    )
    print(doc)


def test_mafengwo_spider():
    spider = AutoSpider()
    doc = spider.crawl("https://www.mafengwo.cn/i/2987983.html")
    print(doc)
    print(doc.page_content)


@pytest.mark.asyncio
async def test_async_common_spider():
    spider = AutoSpider()
    doc = await spider.a_crawl(
        "http://www.360doc.com/content/24/0102/21/9087553_1109645045.shtml"
    )
    print(doc)
    print(doc.page_content)



import re

import html2text
from parsel import Selector

from dataharvest.purifier.purifier import BasePurifier
from dataharvest.schema import Document


class QQNewPurifier(BasePurifier):
    def __init__(self):
        self.convertor = html2text.HTML2Text()
        self.convertor.ignore_links = True
        self.convertor.body_width = 0

    def match(self, url: str) -> bool:
        return "/new.qq.com/rain/a/" in url

    def purify(self, doc: Document) -> Document:
        selector = Selector(doc.page_content)

        # 清洗标签
        selector.xpath("//div[@class='videoPlayer']").drop()
        selector.xpath("//div[@class='qnt-link-list']").drop()

        # 标题
        title_label = selector.xpath("//div[@class='content-article']/h1").get()
        title = self.convertor.handle(title_label)

        # 内容
        content_label = selector.xpath(
            "//div[starts-with(@class, 'article-content-wrap')]"
        ).get()
        content_label_replaced = re.sub(r"<img\b[^>]*?>", "[图片]", content_label)

        content = self.convertor.handle(content_label_replaced)

        return Document(
            url=doc.url, metadata={**doc.metadata}, page_content=title + content
        )
